// ==================================================================
// Apple Metal Shaders
// ==================================================================

#include <simd/simd.h>

using namespace metal;

// Include header shared between this Metal shader code and C code executing Metal API commands
#include "MetalShaderTypes.h"

// Vertex shader outputs and fragment shader inputs
typedef struct
{
  float4 clipSpacePosition [[position]];
  float4 color;
  float pointSize [[point_size]];
  float3 eye;
  float3 normal;
} RasterizerData;

// Vertex function
vertex RasterizerData
vertexShader(uint vertexID [[vertex_id]],
             constant MetalVertex *vertices [[buffer(MetalVertexInputIndexVertices)]],
             constant vector_uint2 *viewportSizePointer [[buffer(MetalVertexInputIndexViewportSize)]],
             constant float4x4 &mvp [[buffer(2)]])
{
  RasterizerData out;
  out.pointSize = 10;
  // Initialize our output clip space position
  out.clipSpacePosition = vector_float4(0.0, 0.0, 0.0, 1.0);
  vector_float4 tmp = vector_float4(0.0, 0.0, 0.0, 1.0);
  tmp = vertices[vertexID].position;
  tmp = mvp * tmp;
  out.clipSpacePosition = tmp;
  out.color = vertices[vertexID].color;
  return out;
}

// Fragment function
fragment float4 fragmentShader(RasterizerData in [[stage_in]],
                               float2 pointCoord [[point_coord]])
{

  // was suppsed to make the point round but doesn't seem to be working
  if (length(pointCoord - float2(0.5)) > 0.5) {  
    // in.color.b = 0.0;
    // discard_fragment();  
  }  
  
  // We return the color we just set which will be written to our color attachment.
  return in.color;
}
