// ================================================================
// Parse the command line arguments and the input file
// ================================================================

#include <iostream>
#include <fstream>

#include "argparser.h"
#include "ifs.h"

#if __APPLE__
#include "matrix.h"
#else
#include <glm/gtc/type_ptr.hpp>
#endif


// The command line arguments
ArgParser::ArgParser(int argc, const char *argv[], IFS *_ifs) {

  // set some default values
  ifs = _ifs;
  path = ".";
  
  // parse the command line arguments
  for (int i = 1; i < argc; i++) {
    if (std::string(argv[i]) == std::string("-input") || 
        std::string(argv[i]) == std::string("-i")) {
      i++; assert (i < argc); 
      separatePathAndFile(argv[i],path,input_file);
    } else if (argv[i] == std::string("-points")) {
      i++; assert (i < argc); 
      ifs->point_count = atoi(argv[i]);
    } else if (argv[i] == std::string("-iters")) {
      i++; assert (i < argc); 
      ifs->iters = atoi(argv[i]);
    } else if (argv[i] == std::string("-size")) {
      i++; assert (i < argc); 
      ifs->width = ifs->height = atoi(argv[i]);
    } else if (argv[i] == std::string("-cubes")) {
      ifs->cubes = true;
    } else if (argv[i] == std::string("-perspective")) {
      ifs->perspective = true;
    } else {
      std::cout << "ERROR: unknown command line argument " 
                << i << ": '" << argv[i] << "'" << std::endl;
      exit(1);
    }
  }

  // parse the transformations file
  parseInputFile();
}


void ArgParser::separatePathAndFile(const std::string &input, std::string &path, std::string &file) {
  // we need to separate the filename from the path
  // (we assume the vertex & fragment shaders are in the same directory)
  // first, locate the last '/' in the filename
  size_t last = std::string::npos;  
  while (1) {
    int next = input.find('/',last+1);
    if (next != (int)std::string::npos) { 
      last = next;
      continue;
    }
    next = input.find('\\',last+1);
    if (next != (int)std::string::npos) { 
      last = next;
      continue;
    }
    break;
  }
  if (last == std::string::npos) {
    // if there is no directory in the filename
    file = input;
    path = ".";
  } else {
    // separate filename & path
    file = input.substr(last+1,input.size()-last-1);
    path = input.substr(0,last);
  }
}


void ArgParser::parseInputFile() {

  // open the file
  std::ifstream input((path+"/"+input_file).c_str());
  if (!input.good()) {
    std::cout << "ERROR: could not open input file" << path+"/"+input_file << std::endl;
      exit(1);
  }

  // read the number of transforms
  int num_transforms; 
  input >> num_transforms;

  // read in the transforms
  for (int i = 0; i < num_transforms; i++) {
    float probability; 
    input >> probability;

    float16 float_data;
    for (int row = 0; row < 4; row++) {
      for (int col = 0; col < 4; col++) {
        input >> float_data.data[col*4+row];
      }
    }


    // ASSIGNMENT: do something with each probability & matrix
    

  } 
}
