#include <string.h>
#include <string.h>
#include <cassert>
#include <iostream>
#include <fstream>
#include <random>

#include "matrix.h"
#include "ifs.h"

double gen_rand_float() {
  static std::random_device rd;    
  static std::mt19937 engine(rd());
  static std::uniform_real_distribution<double> dist(0.0, 1.0);
  return dist(engine);
}

// ====================================================================

// position & color of vertices for the 12 triangles used to draw a unit cube.
const float8 cube[12*3] =
  {
   // back face, cyan
   { 0,0,0, 1,     0,1,1, 1 },
   { 0,1,0, 1,     0,1,1, 1 },
   { 1,1,0, 1,     0,1,1, 1 },
   { 0,0,0, 1,     0,1,1, 1 },
   { 1,1,0, 1,     0,1,1, 1 },
   { 1,0,0, 1,     0,1,1, 1 },
   // front face, red
   { 0,0,1, 1,     1,0,0, 1 },
   { 1,1,1, 1,     1,0,0, 1 },
   { 0,1,1, 1,     1,0,0, 1 },
   { 0,0,1, 1,     1,0,0, 1 },
   { 1,0,1, 1,     1,0,0, 1 },
   { 1,1,1, 1,     1,0,0, 1 },
   // bottom face, purple
   { 0,0,0, 1,     1,0,1, 1 },
   { 1,0,1, 1,     1,0,1, 1 },
   { 0,0,1, 1,     1,0,1, 1 },
   { 0,0,0, 1,     1,0,1, 1 },
   { 1,0,0, 1,     1,0,1, 1 },
   { 1,0,1, 1,     1,0,1, 1 },
   // top face, green
   { 0,1,0, 1,     0,1,0, 1 },
   { 0,1,1, 1,     0,1,0, 1 },
   { 1,1,1, 1,     0,1,0, 1 },
   { 0,1,0, 1,     0,1,0, 1 },
   { 1,1,1, 1,     0,1,0, 1 },
   { 1,1,0, 1,     0,1,0, 1 },
   // left face, yellow
   { 0,0,0, 1,     1,1,0, 1 },
   { 0,0,1, 1,     1,1,0, 1 },
   { 0,1,1, 1,     1,1,0, 1 },
   { 0,0,0, 1,     1,1,0, 1 },
   { 0,1,1, 1,     1,1,0, 1 },
   { 0,1,0, 1,     1,1,0, 1 },
   // right face, blue
   { 1,0,0, 1,     0,0,1, 1 },
   { 1,1,1, 1,     0,0,1, 1 },
   { 1,0,1, 1,     0,0,1, 1 },
   { 1,0,0, 1,     0,0,1, 1 },
   { 1,1,0, 1,     0,0,1, 1 },
   { 1,1,1, 1,     0,0,1, 1 }
  };


// ====================================================================
// ====================================================================


// default values for the IFS variables
void INIT_IFS(IFS *ifs) {


  // ASSIGNMENT: initialize any new variables as needed

  
  ifs->point_count = 10000;
  ifs->cube_count = 0;

  ifs->iters = 3;
  ifs->width = 400;
  ifs->height = 400;

  ifs->cubes = false;
  ifs->perspective = false;

  ifs->pointData = NULL;
  ifs->cubeData = NULL;
}



void generatePoints(IFS *ifs) {
  // clean out any existing point data...
  delete [] ifs->pointData;

  ifs->pointData = new float[ifs->point_count*8];

  for (int i = 0; i < ifs->point_count*8; i+=8) {

    // generate points inside the unit cube (0,0,0)->(1,1,1)
    double x = gen_rand_float();
    double y = gen_rand_float();
    double z = gen_rand_float();
    Vec4f pt(x,y,z,1);


    // ASSIGNMENT: manipulate point position


    // position
    ifs->pointData[i+0] = pt.x();
    ifs->pointData[i+1] = pt.y();
    ifs->pointData[i+2] = pt.z();
    ifs->pointData[i+3] = 1;

    // color (black)
    ifs->pointData[i+4] = 0;
    ifs->pointData[i+5] = 0;
    ifs->pointData[i+6] = 0;
    ifs->pointData[i+7] = 1;  // alpha
  }
}


void generateCubes(IFS *ifs) {
  // clean out any existing cube data...
  delete [] ifs->cubeData;


  // ASSIGNMENT: manipulate vertices of cube (and make more cubes)
  

  // just draw 1 cube for now
  ifs->cube_count = 1;
  ifs->cubeData = new float[8*ifs->cube_count*3*12];
  float* current = ifs->cubeData;
  
  memcpy(current, &cube, sizeof(float)*8*3*12);
  

}


// ASSIGNMENT: add whatever additional functions you need



// ====================================================================
// ====================================================================

